@@ -4,6 +4,6 @@ use MooX::Purple::G -lib => 't/test', -prefix => 'Foo', -module => 1;
 use Foo::Roles;
 class +Class with qw/~Role -One -Two -Three -Four/ {
 	public print {
-		return $_[1];
-	}
+		return $_[0];
+	} describe q(This is a description for the print method.) 
 }
